/*
 *	boost.c		FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "sys.h"
#include "intr.h"
#include "disp.h"
#include "ecu.h"

void		disp_boost( unsigned short w);

char		md_Boost( char next, char enter )
{
	short	t;
	char		c_titled, b_captured, b_cmd;
	unsigned char		btn;
	unsigned short	w, old, uc_peak;

	uc_tmr25 = 0;									/* 25ms timer */
	t = s_min; c_titled = 0; uc_comitval = 0;
	b_captured = 0; b_cmd = 0;
	uc_peak = 0; w = 0xffff; old = 0xffff;

	disp_cls();
	disp_kanji(K16_BOOST);

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if((btn & BTN_UP) && c_titled == 2){		/* peak hold reset */
			uc_btnmsk = BTNMSKTIME;
			disp_boost(uc_peak);
			putc_5p( 0, 8, 'P');
			putc_5p( 4, 8, 'e');
			putc_5p( 8, 8, 'a');
			putc_5p( 12, 8, 'k');
			uc_tmr25 = 0;
			while( uc_tmr25 < 80 ){					/* wait 2s */
				if((getbtn() & BTN_UP) && c_titled == 2){	/* peak reset if pushed DOWN */
					uc_btnmsk = BTNMSKTIME;
					uc_peak = 0;
					disp_boost(0xffff);				/* display '-' */
					old = 0xffff;
				}
			}
			uc_btnmsk = BTNMSKTIME;
			disp_cls();
			disp_boost(w);
			old = w;
			disp_time(s_min);

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_boost(w);							/* make base form */
			disp_banner( 8, 94, (TEXT)"Boost");
			c_titled = 1;

		} else if( !c_bnrstrp && c_titled == 1){
			disp_time(s_min);
			c_titled = 2;

		} else if(uc_tmr25 > CMDWAIT && !b_cmd){		/* clear title after 1s */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon2( ECU16_BOOSTw, ECU16_BOOSTw + 1))
				continue;
			b_cmd = 1;

		} else if( uc_comitval > 4 && b_cmd ){		/* refresh every 125ms */
			if(!b_captured){
				w = 0xffff;
				disp_boost(w);
				old = w;
			}
			if(ecu_reexec())
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd){		/* capture receive data */
			*((char *)&w + 1) = uc_datas[0];
			*((char *)&w) = uc_datas[1];
			c_comstate = COMS_READY;					/* got datas */
			if(w != 0xffff && w > uc_peak )
				uc_peak = w;
			if( old != w ){							/* refresh disp temp. */
				disp_boost(w);
				old = w;
			}
			b_captured = 1;
		}

		if(t != s_min){
			t = s_min;
			disp_time(t);							/* refresh the time */
		}
	}

	return M_BOOST;
}

#ifdef notdef
char		md_BoostMap( char next, char enter)
{
	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;
			disp_cls();
			disp_boost(w);
			disp_time(s_min);

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;

		}
}
#endif

void		disp_boost(unsigned short w)
{
	unsigned long		v;
	unsigned char		b;

	if( w == 0xffff ){
		putc_5p( 0, 0, ' ');
		putc_5p( 5, 0, ' ');
		putc_5p(10, 0, '-');
		return;
	}

	v = w * (unsigned long)500;
	*((char *)&w ) = *((char *)&v + 1);
	*((char *)&w + 1) = *((char *)&v + 2);

	if( w < 1000 ){					/* neg. pressure */
		w = 1000 - w;
		putc_4p( 0, 0, '-');

		if( w >= 1000 )
			w = 999;
		if( w >= 100 )
			putc_4p( 4, 0, '0' + (unsigned char)(w / (unsigned short)100));
		else
			putc_4p( 4, 0, ' ');

		b = w % (unsigned short)100;
		if( w >= 10 )
			putc_4p( 8, 0, '0' + b / 10);
		else
			putc_4p( 8, 0, ' ');

		putc_4p(12, 0, '0' + b % 10);

	} else {
		v = (w - 1000) * (unsigned long)13276 / (unsigned long)100000;
		b = v;
		putc_4p( 0, 0, '_');
		putc_4p( 2, 0, '0' + b / 100);
		putc_4p( 6, 0, '.');
		putc_4p( 8, 0, '0' + (b / 10) % 10);
		putc_4p(12, 0, '0' + b % 10);
	}
}
