/*
 *	disp.c		FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "sys.h"
#include "font.h"
#include "intr.h"

unsigned char		uc_VRAM[32];
char				c_scanl = 0;

far char			*c_bnrstrp = 0;
char				c_bnrstrs, c_bnr_y;
unsigned char		c_bnrwait, c_bnrdelay, c_bnrstate;

void		banner_shift();
void		banner_cls();

void		C5920_send()
{
	unsigned char		k;

if(SW_IGON == 0){
	DISP_BLNK = 1;
	return;
}
	k = c_scanl << 1;
	SSPBUF = uc_VRAM[k];
	while(!SSPIF);
	SSPIF = 0;
	SSPBUF = uc_VRAM[k + 1];
	while(!SSPIF);
	SSPIF = 0;

	DISP_BLNK = 1;										/* disp blanking */
	LATA &= c_scanl | 0xf0;								/* make common output */
	LATA |= c_scanl & 0x0f;
	DISP_LAT = 1; DISP_LAT = 0;							/* latch register */
	DISP_BLNK = 0;
	if( ++c_scanl > 15 ){
		c_scanl = 0;										/* increase scan line */
		if( c_bnrstrp ){
			c_bnrwait++;
			if( c_bnrstate == 0 ){							/* first wait */
				if( c_bnrwait > c_bnrdelay ){
					c_bnrwait = 0;
					c_bnrstate = 1;
				}
	
			} else if( c_bnrstate == 2 ){					/* tail wait */
				if( c_bnrwait > 24 ){
					c_bnrwait = 0;
					c_bnrstate = 0;
					banner_cls();
				}
			
			} else if( c_bnrwait > 2){
				c_bnrwait = 0;
				banner_shift();
			}
		}
	}
}



char	putc_5p(char x, char y, char ch)
{
	char		i, s, w;
	unsigned char		v, f, va, m;

	if(ch >= 0x20 && ch <= 0x7a)
		ch -= 0x20;
	else
		ch = 0x5b;

	s = x % 8;
	w = font5_09Az[ch][0];
	va = (x >> 3) + (y << 1);
	for( i = 0; i < 8; i++){
		f = font5_09Az[ch][i + 1];
		m = 0xff << (8 - w);
		if( va >= 32 ) va = 32;
		v = uc_VRAM[va];
		v &= ~(m >> s);							/* clear bits */
		v |= f >> s;								/* set bits */
		uc_VRAM[va] = v;

		if(s > (8 - w) && (x + w) < 16){
			if( va >= 31 ) va = 31;
			v = uc_VRAM[va + 1];
			v &= ~(m << (8 - s));
			v |= f << (8 - s);
			uc_VRAM[va + 1] = v;
		}
		va += 2;
	}

	return w;
}


char	putc_4p(char x, char y, char ch)
{
	char		i, s, w;
	unsigned char		v, f, va, m;

	if(ch >= 0x30 && ch <= 0x39)
		ch -= 0x30;

	else if( ch >= 'A' && ch <= 'F')
		ch -= 0x30;

	else {
		switch(ch){
		case ' ':
			ch = 0x0c;
			break;
		case ':':
			ch = 0x0a;
			break;
		case 'l':
			ch = 0x0b;
			break;
		case '_':
			ch = 0x0d;
			break;
		case 'c':
			ch = 0x0e;
			break;
		case '-':
			ch = 0x0f;
			break;
		case '.':
			ch = 0x10;
			break;
		case '>':
			ch = 0x17;
			break;
		default:
			ch = 0x18;
			break;
		}
	}
	s = x % 8;
	w = font4_09[ch][0];
	va = (x >> 3) + (y << 1);
	for( i = 0; i < 8; i++){
		f = font4_09[ch][i + 1];
		m = 0xff << (8 - w);
		if( va >= 32 ) va = 32;
		v = uc_VRAM[va];
		v &= ~(m >> s);							/* clear bits */
		v |= f >> s;								/* set bits */
		uc_VRAM[va] = v;

		if(s > (8 - w) && (x + w) < 16){
			if( va >= 31 ) va = 31;
			v = uc_VRAM[va + 1];
			v &= ~(m << (8 - s));
			v |= f << (8 - s);
			uc_VRAM[va + 1] = v;
		}
		va += 2;
	}

	return w;
}

void	disp_cls()
{
	char	i;
	for(i = 0; i < 32; i++) uc_VRAM[i] = 0;
}


void		disp_time(short t)
{
	char	hour, min;

	hour = t / 60 + 1; min = t % 60;

	if( hour > 9 )
		putc_4p( 0, 8, 'l');
	else
		putc_4p( 0, 8, '_');
	putc_4p( 3, 8, '0' + (hour % 10));
	putc_4p( 7, 8, ':');
	putc_4p( 9, 8, '0' + (min / 10));
	putc_4p(13, 8, '0' + (min % 10));
}


void		disp_altblink( char x, char y, char w, char sw)
{
	char		s;
	unsigned char		v, va, m;

	s = x % 8;
	va = (x >> 3) + (y << 1) + 14;
	m = 0xff << (8 - w);

	if( va >= 32 ) va = 32;
	v = uc_VRAM[va];
	if(sw)
		v |= m >> s;					/* show bar */
	else
		v &= ~( m >> s);				/* clear bar */
	uc_VRAM[va] = v;
	if(s > (8 - w)){
		if( va >= 31 ) va = 31;
		v = uc_VRAM[va + 1];
		if(sw)
			v |= m << (8 - s);
		else
			v &= ~(m << (8 - s));

		uc_VRAM[va + 1] = v;
	}
}


void		disp_dot( char x, char y, char sw)
{
	char		va;

	va = (x >> 3) + (y << 1);
	if(sw)
		uc_VRAM[va] |= 0x80 >> (x % 8);
	else
		uc_VRAM[va] &= ~(0x80 >> (x % 8));
}

void		disp_banner( char y, unsigned char delay, far char *str )
{
	char		x, w, ch;

	x = 0;
	c_bnr_y = y;
	c_bnrstrp = 0;
	while(1){
		ch = *str;
		w = putc_5p( x, y, ch);
		x += w;
		if( x > 15 )
			break;
		str++;
	}
	c_bnrdelay = delay;			/* head string wait */
	c_bnrstate = 0; c_bnrwait = 0;
	c_bnrstrs = w - (x - 16);		/* character bit shift */
	c_bnrstrp = str;				/* string pointer */
}


void		banner_cls()
{
	char		va, i;

	va = c_bnr_y << 1;
	c_bnrstrp = 0;				/* banner operation done */
	for( i = 0; i < 16; i++){		/* clear VRAM */
		if( va + i > 31 )
			break;
		uc_VRAM[va + i] = 0;
	}
}

void		banner_shift()
{
	char		va, i, w;
	unsigned char		m, ch;

	ch = *c_bnrstrp;
	if(ch >= 0x20 && ch <= 0x7a)
		ch -= 0x20;
	else
		ch = 0x5b;

	va = c_bnr_y << 1;
	w = font5_09Az[ch][0];
	m = 0x80 >> c_bnrstrs;
	for(i = 0; i < 8; i++){
		if( va >= 30 )
			break;
		if(!c_bnrstrp)					/* now comes new string, cancel operation */
			return;
		uc_VRAM[va] <<= 1;
		if( uc_VRAM[va + 1] & 0x80 )
			uc_VRAM[va] |= 1;
		uc_VRAM[va + 1] <<= 1;
		if( font5_09Az[ch][i + 1] & m )
			uc_VRAM[va + 1] |= 1;
		va += 2;
	}
	if( ++c_bnrstrs >= w){
		c_bnrstrs = 0;
		c_bnrstrp++;
		if( *c_bnrstrp == 0 ){				/* null char excepted, shift done */
			c_bnrstate = 2;				/* last wait */
			c_bnrwait = 0;
		}
	}
}

void		disp_xorline( char y)
{
	char		i;

	y <<= 1;
	for(i = 0; i < 16; i++){
		if( i + y > 31 )
			break;
		uc_VRAM[y + i] ^= 0xff;
	}
}

void		disp_kanji(char ch)
{
	char		i;
	for( i = 0; i < 32; i++)
		uc_VRAM[i] = font_k16[ch][i];
}
