/*
 *	ecu.c		FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "intr.h"
#include "ecu.h"

char			b_actTEN;
char			c_cntTEN, c_rbufp, c_sbufp, c_sbufl;
unsigned char		uc_rbuf[10], uc_sbuf[10];
unsigned char		uc_commTO;				/* comm. timer */

char			c_nAddrs, c_comstate;

unsigned char		uc_datas[4];


char		ecu_waitdata();

void		uart_send()
{
	TXREG = uc_sbuf[c_sbufp++];
	if(c_sbufp == c_sbufl || c_sbufp > 9){
		TXIE = 0;					/* sending done, off interrupt */
	}
}

void		uart_recv()
{
	char		i, l;

	uc_rbuf[c_rbufp++] = RCREG;
	switch(uc_rbuf[0]){
	case 0x00:						/* NOP ?*/
	case 0x03:						/* reexecure command */
	case 0x12:						/* NOP */
	case 0x13:						/* error */
	case 0x14:						/* checksum error */
	case 0x19:						/* receive timeout */
	case 0x1c:						/* test accepted */
	case 0x1d:						/* done test */
		l = 2;
		break;

	case 0x40:						/* Read Command */
	case 0x41:
	case 0x42:
	case 0x43:
		l = (uc_rbuf[0] & 0x0f) + 1;
		if( c_comstate == COMS_SENDING )
			l = l * 2 + 2;
		else
			l = l + 2;
		break;

	case 0x15:						/* ECU ID */
		l = 10;
		break;

	default:
		l = 10;
		c_comstate = COMS_ERROR;		/* error */
		return;
	}

	if( c_rbufp >= l ){				/* done receive */

		if( (uc_rbuf[0] >= 0x40) && (uc_rbuf[0] <= 0x43)
		 	&& (c_comstate == COMS_SENDING )){
			c_comstate = COMS_WAITQUERY;
			c_rbufp = 0;
			uc_commTO = 0;					/* timeout counter for ECU responce */
			return;

		} else if( (uc_rbuf[0] >= 0x40) && (uc_rbuf[0] <= 0x43)
					&& (c_comstate == COMS_WAITQUERY)){

			for(i = 0; i <= (uc_rbuf[0] & 0x0f); i++){
				uc_datas[i] = uc_rbuf[i+1];
			}
			c_comstate = COMS_DONE;
			return;

		} else if( uc_rbuf[0] == 0x15){			/* ECU ID? */

			c_comstate = COMS_READY;
			return;

		} else if( uc_rbuf[0] == 3){			/* feedback data reexecute, skip */
			c_comstate = COMS_WAITQUERY;
			c_rbufp = 0;
			uc_commTO = 0;
			return;

		} else {
			c_comstate = COMS_ERROR;
			c_rbufp = 0;
			uc_commTO = 0;
			return;
		}
	}

	if( c_rbufp > 9 ) c_rbufp = 9;
}

char		ecu_cominit()
{
	char		d;
	
	TXIE = 0; RCIE = 0;
	d = RCREG; d = RCREG;
	b_actTEN = 1;
	c_sbufl = 0; c_rbufp = 0;
	c_comstate = COMS_LINKING;
	TXIE = 0; RCIE = 1;

	d = 	ecu_waitdata();

	return	d;
}

char		ecu_waitdata()
{
	uc_tmr25 = 0;
	while(c_comstate != COMS_READY){
		if( uc_tmr25 > 200)
			return -1;					/* timeout 5sec */
	}
	return 0;
}

char 	ecu_mon(unsigned short addr)
{
	if( c_comstate != COMS_READY )
		return -1;

	uc_sbuf[0] = 0x40;
	uc_sbuf[1] = *((char *)&addr+1);
	uc_sbuf[2] = *((char *)&addr);
	uc_sbuf[3] = 0x40 + uc_sbuf[1] + uc_sbuf[2];

	c_sbufp = 0; c_rbufp = 0;
	c_sbufl = 4;
	c_comstate = COMS_SENDING;
	TXIE = 1; RCIE = 1;		/* start transmit */
	uc_commTO = 0;			/* responce timer */

	return 0;
}

char		ecu_mon2(unsigned short addr1, unsigned short addr2)
{
	char		i;

	if( c_comstate != COMS_READY)
		return -1;

	uc_sbuf[0] = 0x41;
	uc_sbuf[1] = *((char *)&addr1+1);
	uc_sbuf[2] = *((char *)&addr1);
	uc_sbuf[3] = *((char *)&addr2+1);
	uc_sbuf[4] = *((char *)&addr2);
	uc_sbuf[5] = 0x41;
	for( i = 1; i < 5; i++)
		uc_sbuf[5] += uc_sbuf[i];

	c_sbufp = 0; c_rbufp = 0;
	c_sbufl = 6;
	c_comstate = COMS_SENDING;
	TXIE = 1; RCIE = 1;		/* start transmit */
	uc_commTO = 0;			/* responce timer */

	return 0;
}

char		ecu_mon4(unsigned short addr1, unsigned short addr2,
				 unsigned short addr3, unsigned short addr4 )
{
	char		i;

	if( c_comstate != COMS_READY)
		return -1;

	uc_sbuf[0] = 0x43;
	uc_sbuf[1] = *((char *)&addr1+1);
	uc_sbuf[2] = *((char *)&addr1);
	uc_sbuf[3] = *((char *)&addr2+1);
	uc_sbuf[4] = *((char *)&addr2);
	uc_sbuf[5] = *((char *)&addr3+1);
	uc_sbuf[6] = *((char *)&addr3);
	uc_sbuf[7] = *((char *)&addr4+1);
	uc_sbuf[8] = *((char *)&addr4);
	uc_sbuf[9] = 0x43;
	for( i = 1; i < 9; i++)
		uc_sbuf[9] += uc_sbuf[i];

	c_sbufp = 0; c_rbufp = 0;
	c_sbufl = 10;
	c_comstate = COMS_SENDING;
	TXIE = 1; RCIE = 1;		/* start transmit */
	uc_commTO = 0;			/* responce timer */

	return 0;
}

char		ecu_reexec()
{
	if( c_comstate != COMS_READY && c_comstate != COMS_DONE)
		return -1;

	uc_sbuf[0] = 0x03;		/* continue caputure, re-exec command */
	uc_sbuf[1] = 0x03;

	c_sbufp = 0;c_rbufp = 0; 
	c_sbufl = 2;
	c_comstate = COMS_SENDING;
	TXIE = 1; RCIE = 1;
	uc_commTO = 0;			/* responce timer */

	return 0;
}
