/*
 *	main.c		FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "sys.h"
#include "disp.h"
#include "ecu.h"
#include "intr.h"
#include "scenes.h"

__CONFIG(1, HS & OSCSEN);
__CONFIG(2, BOREN & PWRTEN & WDTDIS);
__CONFIG(4, DEBUGDIS & LVPDIS & STVREN);
//__CONFIG(4, DEBUGEN & LVPDIS & STVREN);				/* for ICD */
//__CONFIG(5, CPALL);
__CONFIG(5, UNPROTECT);

void		io_init();
void		powerdown();

extern unsigned char	uc_wtwarn, uc_itwarn;

void		main()
{
	char		c_scene;
	unsigned char		btn;

	CCP1CON = 0x0b;
	CCPR1H = 0x80;					/* Timer1 1/32768 */
	CCPR1L = 0x00;

	T1CON = 0x8f;
	TMR1H = 0x00;
	TMR1L = 0x00;

	T0CON = 0x80;						/* 1/2PSC */
	TMR0L = 0xcb;						/* Timer0 = 0x10000 - 833(1.5KHz @10MHz) */
	TMR0H = 0xfa;						/*                   1333(@16MHz)*/

	io_init();
	TMR0IE = 1;
	CCP1IE = 1;
	PEIE = 1;						// peripheral interrupt enable
	GIE = 1;							// grobal interrupt enable

	if( SW_IGON == 0)					/* entering LP mode when IG off */
		powerdown();

warmboot:
	uc_tmr25 = 0;
	VDD_PWR = 1;						/* LED power up */
	while( uc_tmr25 < 4 );				/* wait 100ms */
	disp_cls();
	disp_banner( 0, 24, (TEXT)"TempMeterV0.9");
	while(c_bnrstrp);
//	md_Timer(0, 0);
//	if( SW_IGON == 0){				/* entering LP mode when IG off */
//		powerdown();
//		goto warmboot;
//	}
	disp_banner( 0, 24, (TEXT)"Conneting....");
	if(! ecu_cominit()){
		while(c_bnrstrp);
		uc_rbuf[9] = 0;
		disp_banner( 0, 24, (TEXT)uc_rbuf + 1);
		while(c_bnrstrp){
			btn = getbtn();
			if( btn & BTN_DOWN ){
				uc_btnmsk = BTNMSKTIME;
				c_bnrstrp = 0;
				break;
			}
		}
	}

	c_scene = eeprom_read(0);
	if( c_scene >= M_EOF )
		c_scene = 0;
	uc_wtwarn = eeprom_read(1);
	if( uc_wtwarn > 119 )
		uc_wtwarn = 119;
	uc_itwarn = eeprom_read(2);
	if( uc_itwarn > 119 )
		uc_itwarn = 119;

	while(1){
		if( SW_IGON == 0){							/* entering LP mode when IG off */
			powerdown();
			goto warmboot;
		}

		switch(c_scene){								/* md_func( down button, set button) */
		case M_WTEMPTIMER:
			c_bnrstrp = 0;
			c_scene = md_WtempTimer( M_ITEMPTIMER, M_WTEMPWARN);
			break;
		case M_ITEMPTIMER:
			c_bnrstrp = 0;
			c_scene = md_ItempTimer( M_WTEMPITEMP, M_ITEMPWARN);
			break;
		case M_WTEMPITEMP:
			c_bnrstrp = 0;
			c_scene = md_WtempItemp( M_VSPEEDTIMER, M_WTEMPITEMP);
			break;
		case M_VSPEEDTIMER:
			c_bnrstrp = 0;
			c_scene = md_VSpeedTimer( M_BOOST, M_VSPEEDTIMER);
			break;
		case M_BOOST:
			c_bnrstrp = 0;
			c_scene = md_Boost( M_MAPDISP, M_BOOST);
			break;
		case M_MAPDISP:
			c_bnrstrp = 0;
			c_scene = md_MapDisp( M_READMEMORY, M_MAPDISP);
			break;
		case M_READMEMORY:
			c_bnrstrp = 0;
			c_scene = md_ReadMemory( M_TIMER, M_READMEMORY);
			break;
		case M_TIMER:
			c_bnrstrp = 0;
			c_scene = md_Timer( M_WTEMPTIMER, M_TIMERSET);
			break;


		case M_WTEMPWARN:
			c_scene = md_WtempWarn( M_WTEMPTIMER, M_WWARNSET);
			break;
		case M_ITEMPWARN:
			c_scene = md_ItempWarn( M_ITEMPTIMER, M_IWARNSET);
			break;


		case M_WWARNSET:
			c_scene = md_WwarnSet(M_WTEMPTIMER, M_WTEMPTIMER);
			break;
		case M_IWARNSET:
			c_scene = md_IwarnSet(M_ITEMPTIMER, M_ITEMPTIMER);
			break;
		case M_TIMERSET:
			c_scene = md_TimerSet(M_TIMER, M_TIMER);
			break;
		}

		eeprom_write(0, c_scene);						/* backup current scene */
	}
}

void	io_init()
{
	LATA = 0xfe;
	LATB = 0xff;
	LATC = 0xfb;
	TRISA = 0xf0;
	TRISB = 0xf8;
	TRISC = 0x93;
//	LATD = 0;
//	TRISD = 0xfe;
	ADCON1 = 0x06;
	RBPU = 0;

	TXSTA = 0x20;
	RCSTA = 0x90;
//	SPBRG = 159;						/* 976bps @10.00MHz */
	SPBRG = 255;						/* 976bps @16.00MHz */
	
	SSPCON1 = 0x30;
}


void		powerdown()
{
		b_actTEN = 0;

		while(c_sec & 1);							/* wait timer is even */
		GIE = 0;
		PIE1 = 0; PIE2 = 0;
		TMR1IE = 1; PEIE = 1;
//		LATA = 0x00; LATB = 0x00; LATC = 0x40;		/* shutdown I/O for power save */
//		TRISA = 0x00; TRISB = 0x03; TRISC = 0x84;

		LATA = 0x00; LATB = 0x00; LATC = 0x00;		/* shutdown I/O for power save */
		TRISA = 0xff; TRISB = 0xff; TRISC = 0xfb;
		RBPU = 1;
		TMR0ON = 0;								/* TMR0 disable */
		CCP1CON = 0;
		SSPCON1 = 0x00;
		TXSTA = 0x00; RCSTA = 0x00;
		TMR1IF = 0;
		SCS = 1;
wait:
		asm("sleep");								/* sleep until 2sec interruput */
		if(TMR1IF){
			TMR1IF = 0;
			c_sec += 2;
			if(c_sec >= 60){
				if( ++s_min >= 720 )
					s_min = 0;
				c_sec = 0;
			}
		}
		if( SW_IGON )
			goto wakeup;
		goto	wait;
wakeup:
		SCS = 0;									/* exiting LP mode, switch HS mode */
		CCP1CON = 0x0b;
		c_sbufl = 0; c_rbufp = 0; c_sbufl = 0;
		c_cntTEN = 0; c_comstate = COMS_UNLINK;
		io_init();
		TMR0ON = 1;
		PEIE = 1; CCP1IE = 1; TMR0IE = 1; GIE = 1;
}
