/*
 *	meter.c		FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "sys.h"
#include "intr.h"
#include "disp.h"
#include "ecu.h"

unsigned char			uc_itwarn, uc_wtwarn;

void			disp_temp( char y, unsigned char v, unsigned char w);
void			disp_speed( unsigned char v);
unsigned char		disp_pktemp(unsigned char uc_peak, unsigned char w);
unsigned char		warninput(unsigned char v);


char		md_WtempItemp( char next, char enter )
{
	char		c_titled, b_captured, b_cmd;
	unsigned char		w1, w2, btn, i;
	unsigned char 	uc_peak1, uc_peak2;
	unsigned char		o1, o2;

	uc_tmr25 = 0;									/* 25ms timer */
	disp_cls();
	disp_kanji(K16_AIRWATER);

	w1 = 255; w2 = 255;
	o1 = 0; o2 = 0;
	uc_peak1 = 0; uc_peak2 = 0;
	c_titled = 0; uc_comitval = 0;
	b_captured = 0; b_cmd = 0;

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if((btn & BTN_UP) && c_titled == 2){	/* peak hold value */
			uc_btnmsk = BTNMSKTIME;
			disp_cls();
			uc_tmr25 = 0;
			putc_5p( 0, 4, 'P');
			putc_5p( 4, 4, 'e');
			putc_5p( 8, 4, 'a');
			putc_5p( 12, 4, 'k');
			while( uc_tmr25 < 19 );
			disp_temp( 0, w1, uc_wtwarn);
			disp_temp( 8, w2, uc_itwarn);
			o1 = w1; o2 = w2;

			putc_4p( 13, 0, ' ');
			putc_4p( 13, 8, ' ');
			uc_tmr25 = 0; i = 0;
			while( uc_tmr25 < 80 ){			/* wait 2s */
				btn = getbtn();
				if( btn & BTN_UP ){			/* peak reset if pushed DOWN */
					uc_btnmsk = BTNMSKTIME;
					uc_peak1 = 0; uc_peak2 = 0;
					disp_temp( 0, 255, uc_wtwarn);
					disp_temp( 8, 255, uc_itwarn);
					o1 = uc_peak1; o2 = uc_peak2; i = 1;
				}					
				if(!(uc_blink & 3) && !i){
					if(uc_blink & 4){
						putc_4p( 13, 4, '>');
					} else {
						putc_4p( 13, 4, ' ');
					}
					if(!uc_blink)
					uc_blink = 7;
				}
			}
			disp_temp( 0, w1, uc_wtwarn);
			disp_temp( 8, w2, uc_itwarn);

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_cls();
			disp_temp( 0, w1, uc_wtwarn);
			disp_banner( 8, 94, (TEXT)"Air+Water");
			c_titled = 1;

		} else if( !c_bnrstrp && c_titled == 1){
			disp_temp( 8, w2, uc_itwarn);
			o2 = w2;
			c_titled = 2;

		} else if( uc_tmr25 > CMDWAIT && !b_cmd ){			/* clear title after 1s */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon2( ECU16_WTRTEMP, ECU16_AIRTEMP ))		/* request both temp. */
				continue;

			b_cmd = 1;

		} else if( uc_comitval > 39 && b_cmd ){			/* request every 1sec */
			if(!b_captured){
				w1 = 255; w2 = 255;
				disp_temp( 0, w1, uc_wtwarn);
				o1 = w1;
				if(c_titled == 2){
					disp_temp( 8, w2, uc_itwarn);
					o2 = w2;
				}
			}
			if(ecu_reexec())								/* request data */
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd){		/* capture receive data */
			w1 = uc_datas[0];
			w2 = uc_datas[1];
			c_comstate = COMS_READY;						/* got datas */
			if( w1 != 255 && w1 > uc_peak1 )
				uc_peak1 = w1;
			if( w2 != 255 && w2 > uc_peak2 )
				uc_peak1 = w2;

			if( w1 != o1 ){								/* refresh disp */
				disp_temp( 0, w1, uc_wtwarn);
				o1 = w1;
			}
			if( w2 != o2 && c_titled == 2 ){
				disp_temp( 8, w2, uc_itwarn);
				o2 = w2;
			}
			b_captured = 1;
		}
	}

	return M_WTEMPITEMP;
}


char		md_WtempTimer( char next, char enter )
{
	short	t;
	char		c_titled, b_captured, b_cmd;
	unsigned char		w, btn;
	unsigned char		uc_peak, old;

	uc_tmr25 = 0;									/* 25ms timer */
	t = s_min; w = 255; uc_peak = 0;
	old = 255;
	uc_comitval = 0;
	c_titled = 0; b_captured = 0; b_cmd = 0;

	disp_cls();
	disp_kanji(K16_WATER);

	while(1){
		if( SW_IGON == 0)
			break;
		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if((btn & BTN_UP) && c_titled == 2){	/* peak hold value */
			uc_btnmsk = BTNMSKTIME;
			uc_peak = disp_pktemp(uc_peak, uc_wtwarn);
			disp_cls();
			disp_temp( 0, w, uc_wtwarn);
			disp_time(s_min);

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_cls();
			disp_temp( 0, w, uc_wtwarn);					/* make base form */
			disp_banner( 8, 94, (TEXT)"WaterTemp.");
			c_titled = 1;

		} else if( !c_bnrstrp && c_titled == 1){
			disp_time(s_min);
			c_titled = 2;

		} else if( !b_cmd && uc_tmr25 > CMDWAIT){		/* clear title after 500ms */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon( ECU16_WTRTEMP))
				continue;
			b_cmd = 1;

		} else if(uc_comitval > 9 && b_cmd ){
			if(!b_captured){
				w = 255;
				disp_temp( 0, w, uc_wtwarn);
				old = w;
			}
			if(ecu_reexec())
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd){		/* capture receive data */
			w = uc_datas[0];
			c_comstate = COMS_READY;						/* got datas */
			if( w != 255 && w > uc_peak )
				uc_peak = w;

			if( w != old){
				disp_temp( 0, w, uc_wtwarn);				/* refresh disp temp. */
				old = w;
			}
			b_captured = 1;
		}

		if(t != s_min){
			t = s_min;
			disp_time(t);									/* refresh the time */
		}
	}

	return M_WTEMPTIMER;
}

char		md_WtempWarn( char next, char enter )
{
	unsigned char		btn;

	disp_cls();
	disp_banner( 0, 94, (TEXT)"WaterWaning");

	putc_5p( 1, 8, '0' + uc_wtwarn / 100);
	putc_5p( 6, 8, '0' + (uc_wtwarn / 10) % 10);
	putc_5p(11, 8, '0' + uc_wtwarn % 10);
	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;							/* disable button pushed still 200ms */
			return next;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;

			return next;

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}
	}

	return next;
}

char		md_WwarnSet( char next, char enter )
{
	unsigned char		v;

	v = warninput( uc_wtwarn );
	if( v == 0xff )
		return next;

	else if( v > 119 )
		uc_wtwarn = 119;

	else
		uc_wtwarn = v;

	eeprom_write( 1, uc_wtwarn);

	return enter;
}



char		md_ItempTimer( char next, char enter )
{
	short	t;
	char		c_titled, b_captured, b_cmd;
	unsigned char		w, btn;
	unsigned char 	uc_peak, old;

	uc_tmr25 = 0;									/* 25ms timer */
	t = s_min; w = 255; uc_peak = 0;
	old = 255;
	c_titled = 0; uc_comitval = 0;
	b_captured = 0; b_cmd = 0;

	disp_cls();
	disp_kanji(K16_AIR);

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if((btn & BTN_UP) && c_titled == 2){		/* peak hold value */
			uc_btnmsk = BTNMSKTIME;
			uc_peak = disp_pktemp(uc_peak, uc_itwarn);
			disp_cls();
			disp_temp( 0, w, uc_itwarn);
			old = w;
			disp_time(s_min);

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_temp( 0, w, uc_itwarn);					/* make base form */
			disp_banner( 8, 94, (TEXT)"AirTemp.");
			c_titled = 1;
		
		} else if( !c_bnrstrp && c_titled == 1){
			disp_time(s_min);							/* make base form */
			c_titled = 2;

		} else if(uc_tmr25 > CMDWAIT && !b_cmd ){		/* clear title after 1s */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon( ECU16_AIRTEMP))
				continue;
			b_cmd = 1;

		} else if(uc_comitval > 39 && b_cmd){
			if(!b_captured){
				w = 255;
				disp_temp( 0, w, uc_itwarn);
				old = w;
			}
			if(ecu_reexec())
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd){			/* capture receive data */
			w = uc_datas[0];
			c_comstate = COMS_READY;						/* got datas */
			if( w != 255 && w > uc_peak )
				uc_peak = w;

			if( w != old ){								/* refresh disp temp. */
				disp_temp( 0, w, uc_itwarn);
				old = w;
			}
			b_captured = 1;
		}

		if(t != s_min){
			t = s_min;
			disp_time(t);									/* refresh the time */
		}
	}

	return M_ITEMPTIMER;
}


char		md_ItempWarn( char next, char enter )
{
	unsigned char		btn;

	disp_cls();
	disp_banner( 0, 94, (TEXT)"AirWarning");

	putc_5p( 1, 8, '0' + uc_itwarn / 100);
	putc_5p( 6, 8, '0' + (uc_itwarn / 10) % 10);
	putc_5p(11, 8, '0' + uc_itwarn % 10);
	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;							/* disable button pushed still 200ms */
			return next;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;
			return next;

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}
	}

	return next;
}


char		md_IwarnSet( char next, char enter )
{
	unsigned char		v;

	v = warninput( uc_itwarn );
	if( v == 0xff )
		return next;

	else if( v > 119 )
		uc_itwarn = 119;

	else
		uc_itwarn = v;


	eeprom_write( 2, uc_itwarn);
	
	return enter;
}


char		md_Timer( char next, char enter )
{
	unsigned char		btn, c_titled;
	short	t;

	uc_tmr25 = 0;									/* 25ms timer */
	c_titled = 0;

	disp_cls();
	disp_kanji(K16_TIMER);

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;						/* disable button pushed still 200ms */
			return next;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;


		} else if((btn & BTN_SET) && c_titled == 2){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_cls();
			disp_banner( 0, 94, (TEXT)"Timer");
			disp_time( s_min);
			c_titled = 1;
		
		} else if( !c_bnrstrp && c_titled == 1){
			c_titled = 2;

		} else if(t != s_min && c_titled == 2){
			t = s_min;
			disp_time(t);									/* refresh the time */
		}
	}

	return next;
}



char		md_TimerSet( char next, char enter )
{
	short	t;
	unsigned char		v, x, w, d, btn;
	unsigned char		n[3];
	char				hour, min;

	t = s_min;
	hour = t / 60 + 1; min = t % 60;
	d = 0; v = hour;
	n[0] = hour;
	n[1] = min / 10;
	n[2] = min % 10;
	disp_cls();
	disp_time(t);

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;							/* disable button pushed still 200ms */
			if( d == 0 ){
				if( v == 1 )
					v = 12;
				else
					v--;
			} else if( d == 1){
				if( v == 0 )
					v = 5;
				else
					v--;
			} else {
				if( v == 0 )
					v = 9;
				else
					v--;
			}
			n[d] = v;
			switch(d){
			case 0:
				if( n[0] / 10)
					putc_4p( 0, 8, 'l');
				else
					putc_4p( 0, 8, '_');
				putc_4p( 3, 8, '0' + n[0] % 10);
				break;

			case 1:
				putc_4p( 9, 8, '0' + n[1]);
				break;

			case 2:
				putc_4p(13, 8, '0' + n[2]);
				break;
			}

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;
			if( d == 0 ){
				if( ++v > 12 )
					v = 1;
			} else if( d == 1){
				if( ++v > 5)
					v = 0;
			} else {
				if( ++v > 9 )
					v = 0;
			}
			n[d] = v;
			switch(d){
			case 0:
				if( n[0] / 10)
					putc_4p( 0, 8, 'l');
				else
					putc_4p( 0, 8, '_');
				putc_4p( 3, 8, '0' + n[0] % 10);
				break;

			case 1:
				putc_4p( 9, 8, '0' + n[1]);
				break;

			case 2:
				putc_4p(13, 8, '0' + n[2]);
				break;
			}

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			switch(d){
			case 0:
				disp_altblink( 0, 8, 7, 0);
				break;

			case 1:
				disp_altblink( 9, 8, 4, 0);
				break;
			}
			if( ++d > 2 ){
				s_min = (n[0] - 1) * 60 + n[1] * 10 + n[2];
				c_sec = 0;
				return enter;
			}
			v = n[d];
		}
		if(!(uc_blink & 3)){
			switch(d){
			case 0:
				x = 0; w = 7;
				break;
			case 1:
				x = 9; w = 4;
				break;
			case 2:
				x = 13; w = 3;
				break;
			}
			disp_altblink( x, 8, w, uc_blink & 4);
			if(!uc_blink)
				uc_blink = 7;
		}
	}

	return next;
}

char		md_VSpeedTimer( char next, char enter )
{
	short	t;
	char		c_titled, b_captured, b_cmd;
	unsigned char		w, btn, old;

	uc_tmr25 = 0;									/* 25ms timer */
	t = s_min; w = 255;
	old = 255;
	c_titled = 0; uc_comitval = 0;
	b_captured = 0; b_cmd = 0;

	disp_cls();
	disp_kanji(K16_SPEED);

	while(1){
		if( SW_IGON == 0)
			break;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if( btn & BTN_UP ){					/* peak hold reset */
			uc_btnmsk = BTNMSKTIME;

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			return enter;
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_cls();
			disp_speed(w);								/* make base form */
			disp_banner( 8, 94, (TEXT)"Speed");
			c_titled = 1;

		} else if( !c_bnrstrp && c_titled == 1){
			disp_time(s_min);
			c_titled = 2;
			
		} else if(uc_tmr25 > CMDWAIT && !b_cmd ){	/* clear title after 1s */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon( ECU16_VSPEED))
				continue;

			b_cmd = 1;

		} else if( uc_comitval > 7 && b_cmd ){
			if(!b_captured){
				w = 255;
				disp_speed(w);
				old = w;
			}
			if(ecu_reexec())
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd ){	/* capture receive data */
			w = uc_datas[0];
			c_comstate = COMS_READY;					/* got datas */

			b_captured = 1;
			if( old != w){							/* refresh disp temp. */
				disp_speed(w);
				old = w;
			}
		}

		if(t != s_min){
			t = s_min;
			disp_time(t);								/* refresh the time */
		}
	}

	return M_VSPEEDTIMER;
}

unsigned char		disp_pktemp(unsigned char uc_peak, unsigned char w)
{
	disp_temp(0, uc_peak, w);
	putc_5p( 0, 8, 'P');
	putc_5p( 4, 8, 'e');
	putc_5p( 8, 8, 'a');
	putc_5p( 12, 8, 'k');
	uc_tmr25 = 0;
	while( uc_tmr25 < 80 ){			/* wait 2s */
		if( getbtn() & BTN_UP){		/* peak reset if pushed UP */
			uc_btnmsk = BTNMSKTIME;
			uc_peak = 0;
			disp_temp( 0, 255, w);
		}
	}

	return uc_peak;
}

void		disp_temp( char y, unsigned char v, unsigned char w)
{
	unsigned short	t;
	char		b_warndisp;

	b_warndisp = 0;
	putc_4p(13, y, 'c');				/* small 'c' */

	if( v == 255 ){
		putc_4p( 0, y, ' ');
		putc_5p( 3, y, ' ');
		putc_5p( 8, y, '-');
		return;
	}

	t = v * (unsigned short)160;
	v = *((char *)&t + 1);				/* v = v * 160 / 256 */

	if( v <= 40 ){
		v = 40 - v;
		putc_4p( 0, y, '-');

	} else {
		v = v - 40;
		if( v >= 100 )
			putc_4p( 0, y, 'l');
		else
			putc_4p( 0, y, ' ');

		if( v >= w )
			b_warndisp = 1;
	}

	if( v >= 10 )
		putc_5p( 3, y, '0' + (v / 10) % 10);
	else
		putc_5p( 3, y, ' ');

	putc_5p( 8, y, '0' + v % 10);

	if(b_warndisp)
		disp_xorline(y);

}

void		disp_speed( unsigned char v)
{
	unsigned short	s;

	if( v == 255 ){
		putc_5p( 1, 0, ' ');
		putc_5p( 6, 0, ' ');
		putc_5p(11, 0, '-');
		return;
	}
	s = (v * (unsigned short)356) >> 8;
	v = (unsigned char)s;

	if(v >= 100)
		putc_5p( 1, 0, '0' + v / 100);
	else
		putc_5p( 1, 0, ' ');

	if(v >= 10)
		putc_5p( 6, 0, '0' + (v / 10) % 10);
	else
		putc_5p( 6, 0, ' ');

	putc_5p(11, 0, '0' + v % 10);
}

unsigned char		warninput(unsigned char v)
{
	unsigned char		d, btn;
	unsigned char		n[3];

	d = 0;
	n[0] = v / 100;
	n[1] = (v / 10) % 10;
	n[2] = v % 10;
	putc_5p( 1, 8, '0' + n[0]);
	putc_5p( 6, 8, '0' + n[1]);
	putc_5p(11, 8, '0' + n[2]);

	v = n[0];
	while(1){
		if( SW_IGON == 0)
			return 0xff;
		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;							/* disable button pushed still 200ms */
			if( v == 0 ){
				if( d == 0 )
					v = 1;
				else
					v = 9;
			} else {
				v--;
			}
			putc_5p( d * 5 + 1, 8, '0' + v);
			n[d] = v;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;
			if( d == 0 ){
				if( ++v > 1 )
					v = 0;
			} else {
				if( ++v > 9 )
					v = 0;
			}
			putc_5p( d * 5 + 1, 8, '0' + v);
			n[d] = v;

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			putc_5p( d * 5 + 1, 8, '0' + n[d]);
			if( ++d > 2 ){
				v = n[0] * 100 + n[1] * 10 + n[2];

				return v;
			}
			v = n[d];
		}
		if(!(uc_blink & 3)){
			disp_altblink( d * 5 + 1, 8, 5, uc_blink & 4);
			if(!uc_blink)
				uc_blink = 7;
		}
	}

	return 0xff;
}
