/*
 *	readmem.c	FDN Digital Temperature meter for FD3S
 *				Version 1.0 2003(c) Kashi Electronics Designs,
 *				H.Kashima, All Rights Reserved.
 *				PIC18F252 with compiler HiTech-C
 */
#include <pic18.h>
#include "sys.h"
#include "intr.h"
#include "disp.h"
#include "ecu.h"

char		hex2ch( char d );
char		isaddr( unsigned short addr, char mode);
short	hexinput(unsigned short	s);
void		disp_hex( char y, unsigned short v, char word);


char		md_ReadMemory( char next, char enter)
{
	char		c_titled, b_captured, b_cmd, b_lost, c_style;
	unsigned char		btn;
	unsigned short		temp, addr, data, old;

	uc_tmr25 = 0;									/* 25ms timer */
	data = 0; old = 0;
	uc_comitval = 0; b_lost = 1;
	c_titled = 0; b_captured = 0; b_cmd = 0;

	*((char *)&addr) = eeprom_read(3);
	*((char *)&addr + 1) = eeprom_read(4);
	c_style = eeprom_read(5);

	if(!isaddr( addr, 0))
		addr = 0;

	if( c_style > 2)
		c_style = 0;

	disp_cls();
	disp_kanji(K16_READ);

	while(1){
		if( SW_IGON == 0)
			break;
		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;				/* disable button pushed still 200ms */
			return next;

		} else if((btn & BTN_UP) && c_titled == 2){	/* change display style */
			uc_btnmsk = BTNMSKTIME;
			if( ++c_style > 2 )
				c_style = 0;
			eeprom_write( 5, c_style);
refresh:
			c_bnrstrp = 0;
			disp_cls();
			if(c_style == 0){
				disp_hex( 0, addr, 2);
				disp_hex( 8, data, 0);

			} else if( c_style == 1){
				disp_hex( 0, addr, 2);
				disp_hex( 8, data, 1);

			} else {
				disp_hex( 0, addr + 1, 2);
				disp_hex( 8, data, 1);
			}

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			c_bnrstrp = 0;
			disp_cls();
			disp_banner( 8, 94, (TEXT)"AddressSet");
			temp = hexinput(addr);
			if( temp != -1 && isaddr( temp, 0)){
				addr = temp;
				eeprom_write( 3, *((char *)&addr));
				eeprom_write( 4, *((char *)&addr + 1));
			}
			b_cmd = 0; c_titled = 1; b_lost = 1;
			uc_tmr25 = 0;
			goto refresh;
			
		}

		if(!c_titled && uc_tmr25 > CMDWAIT){
			disp_cls();
			disp_hex( 8, 0, 3);							/* make base form '-' */
			disp_banner( 0, 94, (TEXT)"ReadMemory");
			c_titled = 1;

		} else if( !c_bnrstrp && c_titled == 1){		/* display current address */
			disp_hex( 0, addr, 2);	
			c_titled = 2;

		} else if( !b_cmd && uc_tmr25 > CMDWAIT){		/* clear title after 500ms */
			if( c_comstate == COMS_DONE )
				c_comstate = COMS_READY;
			if(ecu_mon2(addr, addr + 1))
				continue;
			b_cmd = 1;

		} else if(uc_comitval > 9 && b_cmd ){
			if(!b_captured && !b_lost ){
				disp_hex( 8, 0, 3);					/* show ' ' */
				b_lost = 1;
			}
			if(ecu_reexec())
				continue;

			uc_comitval = 0;
			b_captured = 0;

		} else if(c_comstate == COMS_DONE && b_cmd){	/* capture receive data */
			*((char *)&data + 1) = uc_datas[0];
			*((char *)&data) = uc_datas[1];
			c_comstate = COMS_READY;					/* got datas */

			if(c_style == 0){							/* word style */
				if( old != data || b_lost ){
					disp_hex( 8, data, 0);				/* refresh disp  */
					old = data;
				}
			} else if( c_style == 1 ){					/* msb */
				data = *((char *)&data + 1);
				if( old != data || b_lost ){
					disp_hex( 8, data, 1);				/* refresh disp  */
					old = data;
				}

			} else {									/* lsb */
				data = *((char *)&data);
				if( old != data || b_lost ){
					disp_hex( 8, data, 1);				/* refresh disp  */
					old = data;
				}
			}
			b_captured = 1;
			b_lost = 0;
		}
	}

	return M_READMEMORY;
}

void		disp_hex( char y, unsigned short v, char style)
{
	char		i, d, ch, x;

	if( style == 3 ){					/* style 3, '-' */
		for( i = 0; i < 16; i++){
			if((i + (y << 1)) > 31)
				break;
			uc_VRAM[(y << 1) + i] = 0;
		}
		putc_5p(11, y, '-');
		return;
	}

	if(style == 0){						/* style 0, '0000' */
		i = 4;
		x = 12;
	} else if(style == 1){				/* style 1, '00' */
		i = 2;
		x = 11;
		putc_5p( 1, y, ' ');
	} else {							/* style 2, '000' */
		i = 3;
		x = 11;
	}

	while(1){
		d = v & (unsigned char)15;
		ch = hex2ch(d);
		if(style == 0){
			putc_4p(x, y, ch);
			x -= 4;
		} else {
			putc_5p(x, y, ch);
			x -= 5;
		}
		v = v >> 4;
		if( --i == 0 )
			break;
	}
}

short	hexinput(unsigned short s)
{
	unsigned char		v, d, btn;
	unsigned char		n[3];

	d = 0;
	n[0] = (*((char *)&s + 1)) & 15;
	n[1] = (*((char *)&s)) >> 4;
	n[2] = (*((char *)&s)) & 15;
	putc_5p( 1, 0, hex2ch(n[0]));
	putc_5p( 6, 0, hex2ch(n[1]));
	putc_5p(11, 0, hex2ch(n[2]));

	v = n[0];
	while(1){
		if( SW_IGON == 0)
			return	-1;

		btn = getbtn();
		if( btn & BTN_DOWN ){
			uc_btnmsk = BTNMSKTIME;							/* disable button pushed still 200ms */
			if( v == 0 )
				v = 15;
			else
				v--;
			putc_5p( d * 5 + 1, 0, hex2ch(v));
			n[d] = v;

		} else if( btn & BTN_UP ){
			uc_btnmsk = BTNMSKTIME;
			if( ++v > 15 )
				v = 0;
			putc_5p( d * 5 + 1, 0, hex2ch(v));
			n[d] = v;

		} else if( btn & BTN_SET ){
			uc_btnmsk = BTNMSKTIME;
			putc_5p( d * 5 + 1, 0, hex2ch(n[d]));
			if( ++d > 2 ){
				*((char *)&s + 1) = n[0];
				*((char *)&s) = n[1] << 4 | n[2];

				return s;
			}
			v = n[d];
		}
		if(!(uc_blink & 3)){
			disp_altblink( d * 5 + 1, 0, 5, uc_blink & 4);
			if(!uc_blink)
				uc_blink = 7;
		}
	}

	return -1;
}

char		hex2ch( char d )
{
	char		ch;

	if( d < 10 ){
		ch = '0' + d;

	} else if( d < 16 ){
		ch = 'A' + d - 10;

	}

	return ch;
}

char		isaddr( unsigned short addr, char mode)
{
	if(mode){
		if((addr > 0x1ff && addr < 0x800) || addr > 0xbff)
			return 0;		
	} else {
		if((addr > 0x8b7 && addr < 0xc02) || addr > 0xcdd)
			return 0;
	}

	return -1;
}
